/*
Lesen Sie die Aufgaben genau durch.
Schreiben bzw. vervollständigen Sie gekennzeichnete Programmteile wie in den Kommentaren beschrieben.
Verändern Sie aber nicht vorgegebene Methodenköpfe oder andere vorgegebene Programmteile!
*/
public class Tree {

    /*
    Aufgabe:
        Objekte der Klasse Tree stellen binäre Suchbäume über ganzen Zahlen mit Knoten vom Typ Node dar.
        Ergänzen Sie fehlende Teile der Klasse entsprechend den Kommentaren an den mit TODO gekennzeichneten Stellen.
        Hinweis: Die Methode 'add' ist bereits vollständig vorgegeben.

    Punkte (maximal 16):
        8 Punkte für removeLeaves,
        8 Punkte für toString.
        Auch für teilweise korrekte Lösungen werden Punkte vergeben.
    */

    class Node {

        // Keine zusätzlichen Variablen definieren.
        private int elem;
        private Node left;
        private Node right;

        // Den Konstruktor nicht verändern.
        private Node(int elem) {
            this.elem = elem;
            this.left = null;
            this.right = null;
        }

        // Rekursive Hilfsmethode für 'add' von 'Tree'.
        void add(int elem) {
            if (elem > this.elem) {
                if (this.left == null) {
                    this.left = new Node(elem);
                } else {
                    this.left.add(elem);
                }
            } else {
                if (this.right == null) {
                    this.right = new Node(elem);
                } else {
                    this.right.add(elem);
                }
            }
        }

        // Rekursive Hilfsmethode für 'removeLeaves' von 'Tree'.
        // Hinweis: Der Rückgabewert (boolean) kann angeben,
        // wann ein Knoten als Blattknoten gilt und somit entfernt werden muss.
        boolean removeLeaves() {
            if(this.left != null && this.right !=null) {
                boolean left = this.left.removeLeaves();
                boolean right = this.right.removeLeaves();
                if(left){
                    this.left = null;
                }
                if(right){
                    this.right = null;
                }
                return false;
            }
            if(this.left != null){
                boolean left = this.left.removeLeaves();
                if(left){
                    this.left = null;
                }
                return false;
            }
            if(this.right != null){
                boolean right = this.right.removeLeaves();
                if(right){
                    this.right = null;
                }
                return false;
            }
            return true;
        }

        @Override
        // Rekursive Hilfsmethode für 'toString' von 'Tree'.
        public String toString() {
            String res = "";
            if(this.right != null){
                res += this.right.toString() +",";
            }
            res+= this.elem;
            if(this.left != null){
                res += "," +  this.left.toString();
            }
            return res;
        }
    }

    // Der Wurzelknoten des Baumes. Bitte nicht verändern, oder zusätzliche
    // Variablen hinzufügen.
    private Node root;

    public boolean empty() {
        return this.root == null;
    }

    // Fügt neue Elemente in absteigend sortierter Reihenfolge in den Baum ein.
    // z.B. Die Aufrufe add(5),add(2),add(7),add(6),add(8),add(1),add(1),add(3)
    // ergeben folgenden Baum:
    //              5
    //           /    \
    //         7        2
    //       /   \     /  \
    //      8     6   3    1
    //                      \
    //                       1
    public void add(int elem) {
        if (empty()) {
            this.root = new Node(elem);
        } else {
            this.root.add(elem);
        }
    }

    // Diese Methode entfernt alle Blätterknoten des Baumes. Wird die Methode
    // auf den oben gezeigten Baum angewendet, dann entsteht folgender Baum:
    //              5
    //           /    \
    //         7        2
    //                    \
    //                     1
    public void removeLeaves(){
        if(this.root != null){
            if(this.root.left == null && this.root.right == null){
                this.root = null;
            } else {
                this.root.removeLeaves();
            }
        }
    }

    // Retourniert alle Elemente des Baumes in aufsteigender Reihenfolge als
    // String.
    // z.B. der Baum enthält 1, 2, 3 and 4, dann ist das Ergebnis:
    // "[1,2,3,4]"
    // Für einen leeren Baum wird "[]" zurückgegeben.
    @Override
    public String toString() {
        if(this.root == null){
            return "[]";
        }
        return "[" + this.root.toString() +"]";
    }

    // Nur zum Testen (geht nicht in die Beurteilung ein).
    public static void main(String[] args) {

        Tree tree = new Tree();
        tree.add(5);
        tree.add(2);
        tree.add(7);
        tree.add(6);
        tree.add(8);
        tree.add(1);
        tree.add(1);
        tree.add(3);

        System.out.println(tree);
        System.out.println();

        tree.removeLeaves();

        System.out.println(tree);
        System.out.println();

        Tree myTree = new Tree();
        System.out.println(myTree);


    }

}


